from django.conf.urls import patterns, url

urlpatterns = patterns('',
    url(r'^notify/$', 'getpaid.backends.webpaypure.views.notify', name='getpaid-webpaypure-notify'),
    url(r'^success/$', 'getpaid.backends.webpaypure.views.success', name='getpaid-webpaypure-success'),
    url(r'^failure/$', 'getpaid.backends.webpaypure.views.failure', name='getpaid-webpaypure-failure'),
)



from django.conf.urls import patterns, url
from getpaid.backends.dummy.views import DummyAuthorizationView

urlpatterns = patterns('',
    url(r'^payment/authorization/(?P<pk>[0-9]+)/$', DummyAuthorizationView.as_view(), name='getpaid-dummy-authorization'),
)



from django.dispatch import Signal

new_payment_query = Signal(providing_args=['order', 'payment'])
new_payment_query.__doc__ = """
Sent to ask for filling Payment object with additional data:
    payment.amount:			total amount of an order
    payment.currency:		amount currency

This data cannot be filled by ``getpaid`` because it is Order structure
agnostic. After filling values just do return.
"""

user_data_query = Signal(providing_args=['order', 'user_data'])
new_payment_query.__doc__ = """
Sent to ask for filling user additional data:
    user_data['email']:		user email
    user_data['lang']:      lang code in ISO 2-char format

This data cannot be filled by ``getpaid`` because it is Order structure
agnostic. After filling values just do return.
"""

new_payment = Signal(providing_args=['order', 'payment'])
new_payment.__doc__ = """Sent after creating new payment."""


payment_status_changed = Signal(providing_args=['old_status', 'new_status'])
payment_status_changed.__doc__ = """Sent when Payment status changes."""


redirecting_to_payment_gateway_signal = Signal(providing_args=['request', 'order', 'payment', 'backend'])
redirecting_to_payment_gateway_signal.__doc__ = """
Sent just a moment before redirecting. A hook for analytics tools.
"""



#
# This work by Patryk Zawadzki is licensed under a Creative Commons
# Attribution-NonCommercial-ShareAlike 2.5 Poland.
#
# taken from:
# http://room-303.com/blog/2010/04/27/django-abstrakcji-ciag-dalszy/
# http://gist.github.com/584106
#


class AbstractMixin(object):
    _classcache = {}

    @classmethod
    def contribute(cls):
        return {}

    @classmethod
    def construct(cls, *args, **kwargs):
        attrs = cls.contribute(*args, **kwargs)
        attrs.update({
            '__module__': cls.__module__,
            'Meta': type('Meta', (), {'abstract': True}),
        })
        key = (args, tuple(kwargs.items()))
        if not key in cls._classcache:
            clsname = ('%s%x' % (cls.__name__, hash(key))) \
                .replace('-', '_')
            cls._classcache[key] = type(clsname, (cls, ), attrs)
        return cls._classcache[key]



import os
import glob
import datetime
import urllib
from collections import OrderedDict
from itertools import starmap
from shutil import copyfile, rmtree
from stat import S_IEXEC
from subprocess import Popen, PIPE
from tempfile import mkdtemp, mkstemp
from contextlib import contextmanager
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from getpaid.backends import PaymentProcessorBase


TBK_PUBLIC_KEY = """
-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAtKe3HHWwRcizAfkbS92V
fQr8cUb94TRjQPzNTqBduvvj65AD5J98Cn1htE3NzOz+PjPRcnfVe53V4f3+YlIb
6nnxyeuYLByiwoPkCmpOFBxNp04/Yh3dxN4xgOANXA37rNbDeO4WIEMG6zbdQMNJ
7RqQUlJSmui8gt3YxtqWBhBVW79qDCYVzxFrv3SH7pRuYEr+cxDvzRylxnJgr6ee
N7gmjoSMqF16f9aGdQ12obzV0A35BqpN6pRFoS/NvICbEeedS9g5gyUHf54a+juB
OV2HH5VJsCCgcb7I7Sio/xXTyP+QjIGJfpukkE8F+ohwRiChZ9jMXofPtuZYZiFQ
/gX08s5Qdpaph65UINP7crYbzpVJdrT2J0etyMcZbEanEkoX8YakLEBpPhyyR7mC
73fWd9sTuBEkG6kzCuG2JAyo6V8eyISnlKDEVd+/6G/Zpb5cUdBCERTYz5gvNoZN
zkuq4isiXh5MOLGs91H8ermuhdQe/lqvXf8Op/EYrAuxcdrZK0orI4LbPdUrC0Jc
Fl02qgXRrSpXo72anOlFc9P0blD4CMevW2+1wvIPA0DaJPsTnwBWOUqcfa7GAFH5
KGs3zCiZ5YTLDlnaps8koSssTVRi7LVT8HhiC5mjBklxmZjBv6ckgQeFWgp18kuU
ve5Elj5HSV7x2PCz8RKB4XcCAwEAAQ==
-----END PUBLIC KEY-----
"""

PRIVADA_PEM = """
-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEAn3HzPC1ZBzCO3edUCf/XJiwj3bzJpjjTi/zBO9O+DDzZCaMp
14aspxQryvJhv8644E19Q+NHfxtz1cxd2wnSYKvay1gJx30ZlTOAkzUj4QMimR16
vomLlQ3T2MAz1znt/PVPVU7T/JOG9R+EbiHNVKa/hUjwJEFVXLQNME97nHoLjb3v
V5yV2aVhmox7b54n6F3UVPHvCsHKbJpXpE+vnLpVmdETbNpFVrDygXyG+mnEvyiO
BLIwEY3XTMrgXvS069groLi5Gg8C5LDaYOWjE9084T4fiWGrHhn2781R1rykunTu
77wiWPuQHMS0+YC7mhnsk8Z/ilD+aWz/vhsgHwIDAQABAoIBAQCM+Nrt4cpNKQmn
+Ne8348CGRS9ACXp6WRg6OCQXO4zM7lRZAminVgZgSQXE6aJR+T9rIWMeG7GWydX
aJGzEEQJZOjV0MkUr+7mk9qiTOGkGHmGlyHnRQU8jDU59vXe3UEl3l5+NmwHbQht
waf9F7XLmoLK/WoVJA6tICRpCl1oQrpziqN+gjdmMpz9i8I1sMFE7+Y7xf+7S2u7
c1MRPUWqgdS9yViQVh3vZi25m5CyKRVnOB0hpNuZ7nrJymtADYSWt9wV2W1fX+MX
UUoYfxyQQvWryHhGdedU7GGAnoEdblUcDkBuAaFmsm1P8K4HQZLWP4v6pYlW2JLa
Zoaerb3BAoGBANCRevl0CLB0HBU7sCs0eN9fTkIEsh3OVIxPSBqDnKsynJrIWovK
cs37Vb6phzdQO3ADoFJvR9ck8+v6Cv0KR8IOFl9wfC4ZoxkKBBeq94ZLN+YhE2PW
KiRFybqcgCtzxKS3MyWgpIcT9xFtHVjlorZ8Jk51fgLZbGzamtLhderVAoGBAMO0
mIiiV4l2vXzu4tFfkpu/GOx/D9/vAic3X9FOky09BNCyuMXMQgI8e3wWsGEZghls
Vg9KDV5EPxAmpumcdPFK2IMACaH41ac7vys3ZD8kMK0INQkuDAcG4YsxMaTwEPo0
p1i3zwwEWwknw1yJkOyozz0EcIzS9NrZZEjnBHEjAoGAQ81XdeqzvHEyg/CQd6sq
NCtubGXMZYYi1C4d2Yi5kKn2YRcK4HDi23V+TWodK+0oNWToZIQKjbVUmn0Bv3rt
EvezbDlMFUx+SfCIng0VRJIFTQmpnQYNUxdg2gpwXC/ZWFa6CNxtQABMjFy1cqXM
PJild1IYseJurgBu3mkvBTUCgYBqA/T1X2woLUis2wPIBAv5juXDh3lkB6eU8uxX
CEe2I+3t2EM781B2wajrKadWkmjluMhN9AGV5UZ8S1P0DStUYwUywdx1/8RNmZIP
qSwHAGXV9jI0zNr7G4Em0/leriWkRM26w6fHjLx8EyxDfsohSbkqBrOptcWqoEUx
MOQ5HQKBgAS4sbddOas2MapuhKU2surEb3Kz3RCIpta4bXgTQMt9wawcZSSpvnfT
zs5sehYvBFszL3MV98Uc50HXMf7gykRCmPRmB9S+f+kiVRvQDHfc9nRNg2XgcotU
KAE16PQM8GihQ0C+EcXHouyud5CRJGfyurokRlH/jY3BiRAG5c+6
-----END RSA PRIVATE KEY-----
"""

TBK_PARAM_TXT = """
<BP_PAGO>
        <TR_NORMAL>
                TBK_TIPO_TRANSACCION#A#50#1
                TBK_MONTO#M#10#1
                TBK_ORDEN_COMPRA#A#26#1
                TBK_ID_SESION#A#61#0
                TBK_URL_FRACASO#A#256#1
                TBK_URL_EXITO#A#256#1
                TBK_MONTO_CUOTA#M#9#0
                TBK_NUMERO_CUOTAS#N#2#0
        </TR_NORMAL>
        <TR_COMPLETA>
                TBK_TIPO_TRANSACCION#A#50#1
                TBK_MONTO#M#10#1
                TBK_ORDEN_COMPRA#A#26#1
                TBK_ID_SESION#A#61#0
                TBK_URL_EXITO#A#256#1
                TBK_URL_FRACASO#A#256#1
                TBK_NUMERO_TARJETA#N#19#1
                TBK_FECHA_EXPIRACION#N#4#1
                TBK_CVV#N#4#1
                TBK_TIPO_PAGO#A#2#1
                TBK_NUMERO_CUOTAS#N#2#0
                TBK_MONTO_CUOTA#M#9#0
        </TR_COMPLETA>
        <TR_MALL>
                TBK_TIPO_TRANSACCION#A#50#1
                TBK_MONTO#M#10#1
                TBK_ORDEN_COMPRA#A#26#1
                TBK_ID_SESION#A#61#1
                TBK_URL_RESULTADO#A#256#1
                TBK_URL_FRACASO#A#256#1
                TBK_NUM_TRX#N#4#1
        </TR_MALL>
        <TR_MALL_COMPLETA>
                TBK_TIPO_TRANSACCION#A#50#1
                TBK_MONTO#M#10#1
                TBK_ORDEN_COMPRA#A#26#1
                TBK_ID_SESION#A#61#1
                TBK_URL_RESULTADO#A#256#1
                TBK_URL_FRACASO#A#256#1
                TBK_NUM_TRX#N#4#1
                TBK_NUMERO_TARJETA#N#19#1
                TBK_FECHA_EXPIRACION#N#4#1
                TBK_CVV#N#4#1
        </TR_MALL_COMPLETA>
        <TR_TASA_INTERES_MAX>
                TBK_TIPO_TRANSACCION#A#50#1
                TBK_URL_EXITO#A#256#1
                TBK_URL_FRACASO#A#256#1
        </TR_TASA_INTERES_MAX>
</BP_PAGO>
<BP_PAGO_MALL>
        <TR_MALL>
                TBK_CODIGO_TIENDA_M#N#12#1
                TBK_ORDEN_TIENDA_M#A#26#1
                TBK_MONTO_TIENDA_M#M#10#1
                TBK_MONTO_CUOTA_M#M#9#0
                TBK_NUMERO_CUOTAS_M#N#2#0
        </TR_MALL>
        <TR_MALL_COMPLETA>
                TBK_CODIGO_TIENDA_M#N#12#1
                TBK_ORDEN_TIENDA_M#A#26#1
                TBK_MONTO_TIENDA_M#M#10#1
                TBK_TIPO_PAGO_M#A#2#1
                TBK_NUMERO_CUOTAS_M#N#2#0
                TBK_MONTO_CUOTA_M#M#9#0
        </TR_MALL_COMPLETA>
</BP_PAGO_MALL>
"""

TBK_TRACE_DAT = """
GLEVEL     = 7

// niveles soportados  LOGINFO  1
//                     LOGERROR 2
//                     LOGFATAL 4
//
//                glebel 0 >>> No genera LOG
//
// combinatorias  glebel 1 >>> LOGINFO
//                glebel 2 >>> LOGERROR
//                glebel 3 >>> LOGINFO  + LOGERROR
//                glebel 4 >>> LOGFATAL
//                glebel 5 >>> LOGINFO  + LOGFATAL
//                glebel 6 >>> LOGERROR + LOGFATAL
//                glebel 7 >>> LOGINFO  + LOGERROR + LOGFATAL
"""


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.webpay'
    BACKEND_NAME = _('Webpay backend')
    BACKEND_ACCEPTED_CURRENCY = ('CLP', 'USD')
    TESTING_COMMERCE_IDS = {'CLP': '597026007976',
                            'USD': '597026007984'}
    TBK_PARAM_TXT = TBK_PARAM_TXT
    TBK_TRACE_DAT = TBK_TRACE_DAT
    TBK_PUBLIC_KEY = TBK_PUBLIC_KEY
    PRIVADA_PEM = PRIVADA_PEM

    @classmethod
    def get_tbk_config(cls, payment_pk, currency='CLP'):
        conf = OrderedDict((('IDCOMERCIO', None),
                            ('MEDCOM', '1'),
                            ('TBK_KEY_ID', '101'),
                            ('PARAMVERIFCOM', '1'),
                            ('URLCGICOM', reverse('getpaid-webpay-resultado', args=[payment_pk])),
                            ('SERVERCOM', cls.get_backend_setting('STATIC_INBOUND_IP')),
                            ('PORTCOM', '80'),
                            ('WHITELISTCOM', 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz 0123456789./:=&?_'),
                            ('HOST', cls.get_backend_setting('STATIC_INBOUND_IP')),
                            ('WPORT', '80'),
                            ('URLCGITRA', '/filtroUnificado/bp_revision.cgi'),
                            ('URLCGIMEDTRA', '/filtroUnificado/bp_validacion.cgi'),
                            ('SERVERTRA', 'https://certificacion.webpay.cl'),
                            ('PORTTRA', '6443'),
                            ('PREFIJO_CONF_TR', 'HTML_'),
                            ('HTML_TR_NORMAL', reverse('getpaid-webpay-close'))))

        certified = cls.get_backend_setting('CERTIFIED', False)
        if certified:
            conf.update({'IDCOMERCIO': cls.get_backend_setting('COMMERCE_ID_{}'.format(currency)),
                         'URLCGITRA': '/cgi-bin/bp_revision.cgi',
                         'URLCGIMEDTRA': '/cgi-bin/bp_validacion.cgi',
                         'SERVERTRA': 'https://webpay.transbank.cl',
                         'PORTTRA': '443'})
        else:
            conf['IDCOMERCIO'] = cls.TESTING_COMMERCE_IDS[currency]

        return "\n".join(starmap("{} = {}".format, conf.items()))

    @classmethod
    def get_tbk_param(cls):
        return cls.TBK_PARAM_TXT

    @classmethod
    def get_tbk_trace(cls):
        return cls.TBK_TRACE_DAT

    @classmethod
    def get_keys(cls):
        if cls.get_backend_setting('CERTIFIED', False):
            tbk_public_key = "\n" + cls.get_backend_setting('TRANSBANK_PUBLIC_KEY') + "\n"
            commerce_private_key = "\n" + cls.get_backend_setting('COMMERCE_PRIVATE_KEY') + "\n"
            return (tbk_public_key, commerce_private_key)
        else:
            return (cls.TBK_PUBLIC_KEY, cls.PRIVADA_PEM)

    @classmethod
    def validate(cls, payment, request):
        """Validates a Webpay transaction.

        Checks that:
            - The payment amount has not been tampered with.
            - The MAC is authentic (with help of tbk_check_mac.cgi).
            - The payment has yet to be paid.

        Args:
            payment: A getpaid.models.Payment instance.
            request: A django.http.HttpRequest instance.

        Returns:
            A bool.
        """
        amount = str(int(payment.amount)) + '00'
        tbk_amount = request.POST.get('TBK_MONTO', None)
        same_amount = amount == tbk_amount

        temp_fd, temp_file = mkstemp()

        with open(temp_file, 'w') as params_file:
            params_file.write(urllib.unquote(request.body))

        with webpay_run('tbk_check_mac.cgi', payment.pk, temp_file) as cgi:
            output, _ = cgi.communicate()
            os.close(temp_fd)
            os.unlink(temp_file)
            valid_mac = output == 'CORRECTO\n'

        return same_amount and valid_mac and not payment.paid_on

    def get_gateway_url(self, request):
        """Returns the url the user will be redirected to.

        Webpay requires that the request to ``pago`` url must be via POST.
        ``tbk_params`` is returned as context to populate a hidden form that
        auto-submits to ``pago`` url.
        """
        order = self.payment.order
        base_url = settings.SITE_URL

        tbk_params = {'TBK_MONTO': str(order.total) + '00',
                      'TBK_TIPO_TRANSACCION': 'TR_NORMAL',
                      'TBK_ORDEN_COMPRA': order.pk,
                      'TBK_ID_SESION': self.payment.pk,
                      'TBK_URL_EXITO': base_url + reverse('getpaid-webpay-success'),
                      'TBK_URL_FRACASO': base_url + reverse('getpaid-webpay-failure'),}

        return reverse('getpaid-webpay-pago', kwargs={'pk': self.payment.pk}), "POST", tbk_params


@contextmanager
def webpay_run(name, payment_pk, *args, **kwargs):
    """Runs a Webpay binary within a preconfigured environment.

    Before running the binary, this context manager recreates the
    directories and files in /tmp that the binary needs to run correctly,
    as if it were sitting in an standalone web server.

    Yields a subprocess.Popen instance with an open pipe to stdin and stdout.

    After running the binary, log and journal entries are captured and
    then temporary files are removed.

    Args:
        name: A string, basename of the binary as is found in assets_dir.
        payment_pk: An integer, primary key for the related payment. This
            is needed to associate the logs with their payment.
        *args: Extra positional arguments are appended to the command line
            before execution.

    Example:
        >>> webpay_run('tbk_bp_resultado.cgi', 3958) as cgi:
                output, _ = cgi.communicate("TBK_MONTO=384800&...\n")
                do_something_with(output)

    WARNING: Always use Popen with communicate() method when using PIPE
        or there will be deadlocks.
    """

    # prepare the configuration files
    assets_dir = PaymentProcessor.get_backend_setting('ASSETS_DIR')
    tbk_config = PaymentProcessor.get_tbk_config(payment_pk, 'CLP')  # FIXME
    tbk_param = PaymentProcessor.get_tbk_param()
    tbk_trace = PaymentProcessor.get_tbk_trace()

    temp_dir = mkdtemp()
    cgi_path = os.path.join(assets_dir, name)
    temp_cgi_path = os.path.join(temp_dir, name)
    datos_path = os.path.join(temp_dir, 'datos')

    os.mkdir(datos_path)
    with open(os.path.join(datos_path, 'tbk_config.dat'), 'w') as f:
        f.write(tbk_config)
    with open(os.path.join(datos_path, 'tbk_param.txt'), 'w') as f:
        f.write(tbk_param)
    with open(os.path.join(datos_path, 'tbk_trace.dat'), 'w') as f:
        f.write(tbk_trace)

    # prepare the public and private keys
    maestros_path = os.path.join(temp_dir, 'maestros')
    public_key, private_key = PaymentProcessor.get_keys()

    os.mkdir(maestros_path)
    with open(os.path.join(maestros_path, 'tbk_public_key.pem'), 'w') as f:
        f.write(public_key)
    with open(os.path.join(maestros_path, 'privada.pem'), 'w') as f:
        f.write(private_key)

    # prepare the log directory
    log_path = os.path.join(temp_dir, 'log')
    os.mkdir(log_path)

    # copy the binary to the temp dir and make it executable
    copyfile(cgi_path, temp_cgi_path)
    os.chmod(temp_cgi_path, S_IEXEC)

    yield Popen([temp_cgi_path] + list(args), stdin=PIPE, stdout=PIPE)

    # capture the logs
    try:
        from getpaid.models import Payment

        payment = Payment.objects.get(pk=payment_pk)

        for event_log in glob.glob(os.path.join(log_path, 'TBK_EVN*')):
            with open(event_log, 'r') as f:
                for line in map(str.strip, f.readlines()):
                    from models import LogEntry

                    entry = LogEntry.from_line(line=line, payment=payment)
                    entry.save()

        for journal_log in glob.glob(os.path.join(log_path, 'tbk_bitacora_TR_NORMAL*')):
            st = os.stat(journal_log)
            date = datetime.date.fromtimestamp(st.st_mtime)
            with open(journal_log, 'r') as f:
                for line in map(str.strip, f.readlines()):
                    from models import JournalEntry
                    entry = JournalEntry(date=date, body=line, payment=payment)
                    entry.save()
    except Payment.DoesNotExist:
        pass

    # clean up
    rmtree(temp_dir)



class SetRemoteAddrFromForwardedForMiddleware(object):
    def process_request(self, request):
        try:
            real_ip = request.META['HTTP_X_FORWARDED_FOR']
        except KeyError:
            pass
        else:
            # HTTP_X_FORWARDED_FOR can be a comma-separated list of IPs.
            # Take just the first one.
            real_ip = real_ip.split(",")[0]
            request.META['REMOTE_ADDR'] = real_ip



# coding: utf-8
import datetime
from functools import partial
from django.db import models


class LogEntry(models.Model):
    magic1 = models.CharField(max_length=10, blank=True, null=True)  # transaction_id
    pid = models.PositiveIntegerField(default=0)
    magic2 = models.CharField(max_length=3, blank=True, null=True)
    action = models.CharField(max_length=10)
    magic3 = models.CharField(max_length=40, blank=True, null=True)
    date = models.DateField()
    time = models.TimeField()
    ip_address = models.CharField(max_length=15, blank=True, null=True)
    status = models.CharField(max_length=3)
    magic4 = models.CharField(max_length=20)
    message = models.TextField()
    payment = models.ForeignKey('getpaid.Payment')

    PROPERTIES = ('magic1', 'pid', 'magic2', 'action', 'magic3', 'date_str',
                  'time_str', 'ip', 'status', 'magic4', 'message')

    @property
    def date_str(self):
        return self.date.strftime("%d%m%Y")

    @date_str.setter
    def date_str(self, value):
        self.date = datetime.date(*map(int, (value[4:], value[2:4], value[:2])))

    @property
    def time_str(self):
        return self.time.strftime("%H%M%S")

    @time_str.setter
    def time_str(self, value):
        self.time = datetime.time(*map(int, (value[:2], value[2:4], value[4:])))

    @property
    def ip(self):
        return self.ip_address or "EMPTY"

    @ip.setter
    def ip(self, value):
        self.ip_address = value

    def __unicode__(self):
        template = "{:<10};{:>12};{:<3};{:<10};{:<40};{:<14};{:<6};{:<15};{:<3};{:<20};{}"
        properties = self.PROPERTIES
        return template.format(*map(partial(getattr, self), properties))

    @classmethod
    def from_line(cls, *args, **kwargs):
        line = kwargs.pop('line')
        entry = cls(**kwargs)
        properties = cls.PROPERTIES
        for property, value in zip(properties, map(str.strip, line.split(';'))):
            setattr(entry, property, value)
        return entry


class JournalEntry(models.Model):
    date = models.DateField()
    body = models.TextField()
    payment = models.OneToOneField('getpaid.Payment')

    @property
    def params(self):
        raw_params = self.body.split('#')
        return dict(map(lambda s: s.split('='), raw_params))

    def __unicode__(self):
        return self.body


def build_models(payment_class):
    """
    Here you can dynamically build a Model class that needs to have
    ForeignKey to Payment model
    """
    return []



from django.conf.urls import patterns, url, include
from getpaid.views import NewPaymentView, FallbackView
from getpaid.utils import import_backend_modules

includes_list = []
for backend_name, urls in import_backend_modules('urls').items():
    includes_list.append(url(r'^%s/' % backend_name, include(urls)))

urlpatterns = patterns('',
    url(r'^new/payment/(?P<currency>[A-Z]{3})/$', NewPaymentView.as_view(), name='getpaid-new-payment'),
    url(r'^payment/success/(?P<pk>\d+)/$', FallbackView.as_view(success=True), name='getpaid-success-fallback'),
    url(r'^payment/failure/(?P<pk>\d+)$', FallbackView.as_view(success=False), name='getpaid-failure-fallback'),
    *includes_list

)



from django.forms import forms
from django.forms.fields import ChoiceField
from django.utils.translation import ugettext as _


class DummyQuestionForm(forms.Form):
    """
    This dummy form asks for payment authorization.
    """
    authorize_payment = ChoiceField(label=_("authorization"), choices=((1, _('yes')), (0, _('no'))))



import rsa
import random
import base64
import hashlib
import datetime
import requests
from Crypto import Random
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.PublicKey import RSA
from collections import OrderedDict
from operator import methodcaller
from itertools import starmap
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from getpaid.backends import PaymentProcessorBase
from getpaid import models as getpaid_models
from models import JournalEntry

TBK_PUBLIC_KEY = """
-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAtKe3HHWwRcizAfkbS92V
fQr8cUb94TRjQPzNTqBduvvj65AD5J98Cn1htE3NzOz+PjPRcnfVe53V4f3+YlIb
6nnxyeuYLByiwoPkCmpOFBxNp04/Yh3dxN4xgOANXA37rNbDeO4WIEMG6zbdQMNJ
7RqQUlJSmui8gt3YxtqWBhBVW79qDCYVzxFrv3SH7pRuYEr+cxDvzRylxnJgr6ee
N7gmjoSMqF16f9aGdQ12obzV0A35BqpN6pRFoS/NvICbEeedS9g5gyUHf54a+juB
OV2HH5VJsCCgcb7I7Sio/xXTyP+QjIGJfpukkE8F+ohwRiChZ9jMXofPtuZYZiFQ
/gX08s5Qdpaph65UINP7crYbzpVJdrT2J0etyMcZbEanEkoX8YakLEBpPhyyR7mC
73fWd9sTuBEkG6kzCuG2JAyo6V8eyISnlKDEVd+/6G/Zpb5cUdBCERTYz5gvNoZN
zkuq4isiXh5MOLGs91H8ermuhdQe/lqvXf8Op/EYrAuxcdrZK0orI4LbPdUrC0Jc
Fl02qgXRrSpXo72anOlFc9P0blD4CMevW2+1wvIPA0DaJPsTnwBWOUqcfa7GAFH5
KGs3zCiZ5YTLDlnaps8koSssTVRi7LVT8HhiC5mjBklxmZjBv6ckgQeFWgp18kuU
ve5Elj5HSV7x2PCz8RKB4XcCAwEAAQ==
-----END PUBLIC KEY-----
"""

PRIVADA_PEM = """
-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEAn3HzPC1ZBzCO3edUCf/XJiwj3bzJpjjTi/zBO9O+DDzZCaMp
14aspxQryvJhv8644E19Q+NHfxtz1cxd2wnSYKvay1gJx30ZlTOAkzUj4QMimR16
vomLlQ3T2MAz1znt/PVPVU7T/JOG9R+EbiHNVKa/hUjwJEFVXLQNME97nHoLjb3v
V5yV2aVhmox7b54n6F3UVPHvCsHKbJpXpE+vnLpVmdETbNpFVrDygXyG+mnEvyiO
BLIwEY3XTMrgXvS069groLi5Gg8C5LDaYOWjE9084T4fiWGrHhn2781R1rykunTu
77wiWPuQHMS0+YC7mhnsk8Z/ilD+aWz/vhsgHwIDAQABAoIBAQCM+Nrt4cpNKQmn
+Ne8348CGRS9ACXp6WRg6OCQXO4zM7lRZAminVgZgSQXE6aJR+T9rIWMeG7GWydX
aJGzEEQJZOjV0MkUr+7mk9qiTOGkGHmGlyHnRQU8jDU59vXe3UEl3l5+NmwHbQht
waf9F7XLmoLK/WoVJA6tICRpCl1oQrpziqN+gjdmMpz9i8I1sMFE7+Y7xf+7S2u7
c1MRPUWqgdS9yViQVh3vZi25m5CyKRVnOB0hpNuZ7nrJymtADYSWt9wV2W1fX+MX
UUoYfxyQQvWryHhGdedU7GGAnoEdblUcDkBuAaFmsm1P8K4HQZLWP4v6pYlW2JLa
Zoaerb3BAoGBANCRevl0CLB0HBU7sCs0eN9fTkIEsh3OVIxPSBqDnKsynJrIWovK
cs37Vb6phzdQO3ADoFJvR9ck8+v6Cv0KR8IOFl9wfC4ZoxkKBBeq94ZLN+YhE2PW
KiRFybqcgCtzxKS3MyWgpIcT9xFtHVjlorZ8Jk51fgLZbGzamtLhderVAoGBAMO0
mIiiV4l2vXzu4tFfkpu/GOx/D9/vAic3X9FOky09BNCyuMXMQgI8e3wWsGEZghls
Vg9KDV5EPxAmpumcdPFK2IMACaH41ac7vys3ZD8kMK0INQkuDAcG4YsxMaTwEPo0
p1i3zwwEWwknw1yJkOyozz0EcIzS9NrZZEjnBHEjAoGAQ81XdeqzvHEyg/CQd6sq
NCtubGXMZYYi1C4d2Yi5kKn2YRcK4HDi23V+TWodK+0oNWToZIQKjbVUmn0Bv3rt
EvezbDlMFUx+SfCIng0VRJIFTQmpnQYNUxdg2gpwXC/ZWFa6CNxtQABMjFy1cqXM
PJild1IYseJurgBu3mkvBTUCgYBqA/T1X2woLUis2wPIBAv5juXDh3lkB6eU8uxX
CEe2I+3t2EM781B2wajrKadWkmjluMhN9AGV5UZ8S1P0DStUYwUywdx1/8RNmZIP
qSwHAGXV9jI0zNr7G4Em0/leriWkRM26w6fHjLx8EyxDfsohSbkqBrOptcWqoEUx
MOQ5HQKBgAS4sbddOas2MapuhKU2surEb3Kz3RCIpta4bXgTQMt9wawcZSSpvnfT
zs5sehYvBFszL3MV98Uc50HXMf7gykRCmPRmB9S+f+kiVRvQDHfc9nRNg2XgcotU
KAE16PQM8GihQ0C+EcXHouyud5CRJGfyurokRlH/jY3BiRAG5c+6
-----END RSA PRIVATE KEY-----
"""


def aes_cbc_encrypt(message, key=None, key_size=256):
    def pad(s):
        x = AES.block_size - len(s) % AES.block_size
        return s + (chr(x) * x)
 
    padded_message = pad(message)
 
    if key is None:
        key = Random.OSRNG.posix.new().read(key_size // 8)
 
    iv = Random.OSRNG.posix.new().read(AES.block_size)
    cipher = AES.new(key, AES.MODE_CBC, iv)

    return (iv, key, cipher.encrypt(padded_message))

def aes_cbc_decrypt(ciphertext, key):
    unpad = lambda s: s[:-ord(s[-1])]
    iv = ciphertext[:AES.block_size]
    cipher = AES.new(key, AES.MODE_CBC, iv)
    plaintext = unpad(cipher.decrypt(ciphertext))[AES.block_size:]

    return plaintext


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.webpaypure'
    BACKEND_NAME = _('Webpay backend in pure Python')
    BACKEND_ACCEPTED_CURRENCY = ('CLP', 'USD')
    TESTING_COMMERCE_IDS = {'CLP': '597026007976',
                            'USD': '597026007984'}
    TBK_PUBLIC_KEY = TBK_PUBLIC_KEY
    PRIVADA_PEM = PRIVADA_PEM

    def get_params(self):
        payment = self.payment
        order = payment.order
        currency = payment.currency
        base_url = settings.SITE_URL
        certified = self.get_backend_setting('CERTIFIED', False)

        if certified:
            commerce_code = self.get_backend_setting('COMMERCE_ID_{}'.format(currency))
        else:
            commerce_code = self.TESTING_COMMERCE_IDS[currency]

        return OrderedDict((('TBK_ORDEN_COMPRA', order.id),
                            ('TBK_CODIGO_COMERCIO', commerce_code),
                            ('TBK_ID_TRANSACCION', int(random.random() * 10000000000)),
                            ('TBK_URL_CGI_COMERCIO', reverse('getpaid-webpaypure-notify')),
                            ('TBK_SERVIDOR_COMERCIO', self.get_backend_setting('STATIC_INBOUND_IP')),
                            ('TBK_PUERTO_COMERCIO', '80'),
                            ('TBK_VERSION_KCC', '6.0'),
                            ('TBK_KEY_ID', '101'),
                            ('PARAMVERIFCOM', '1'),
                            ('TBK_MAC', None),
                            ('TBK_MONTO', str(order.total) + '00'),
                            ('TBK_ID_SESION', payment.id),
                            ('TBK_URL_EXITO', base_url + reverse('getpaid-webpaypure-success')),
                            ('TBK_URL_FRACASO', base_url + reverse('getpaid-webpaypure-failure')),
                            ('TBK_TIPO_TRANSACCION', 'TR_NORMAL')))

    @classmethod
    def get_keys_raw(cls):
        if cls.get_backend_setting('CERTIFIED', False):
            # TODO: End this new line spree
            tbk_public_key = "\n" + cls.get_backend_setting('TRANSBANK_PUBLIC_KEY') + "\n"
            commerce_private_key = "\n" + cls.get_backend_setting('COMMERCE_PRIVATE_KEY') + "\n"
            return (tbk_public_key, commerce_private_key)
        else:
            return (cls.TBK_PUBLIC_KEY, cls.PRIVADA_PEM)

    @classmethod
    def encrypt(cls, message):
        public_key_data, private_key_data = cls.get_keys_raw()

        private_rsa_key = rsa.PrivateKey.load_pkcs1(private_key_data)
        signature = rsa.sign(message, private_rsa_key, 'SHA-512')
        iv, aes_key, encrypted_message = aes_cbc_encrypt(signature + message)
        
        public_key = RSA.importKey(public_key_data.strip())
        public_cipher = PKCS1_OAEP.new(public_key)
        encrypted_key = public_cipher.encrypt(aes_key)

        return base64.b64encode(iv + encrypted_key + encrypted_message)

    @classmethod
    def decrypt(cls, encrypted_message):
        public_key_data, private_key_data = cls.get_keys_raw()

        private_key = RSA.importKey(private_key_data.strip())
        cipher = PKCS1_OAEP.new(private_key)

        public_key = rsa.PublicKey.load_pkcs1_openssl_pem(public_key_data)

        data = base64.b64decode(encrypted_message.strip())
        iv = data[0:16]

        encrypted_key = data[16:272]

        key = cipher.decrypt(encrypted_key)

        signed_message = aes_cbc_decrypt(iv + data[272:], key)
        signature, message = signed_message[0:512], signed_message[512:]

        if rsa.verify(message, signature, public_key):
            return message

    def get_token(self):
        params = self.get_params()

        params['TBK_MAC'] = hashlib.md5("&".join(starmap("{}={}".format, filter(lambda param: param[0] != 'TBK_MAC', params.items()))) + params['TBK_CODIGO_COMERCIO'] + 'webpay').hexdigest()  # :facepalm:

        if self.get_backend_setting('CERTIFIED', False):
            token_endpoint_url = 'https://webpay.transbank.cl:443/cgi-bin/bp_validacion.cgi'
        else:
            token_endpoint_url = 'https://certificacion.webpay.cl:6443/webpayserver/bp_validacion.cgi'

        data = {'TBK_VERSION_KCC': '6.0',
                'TBK_CODIGO_COMERCIO': params['TBK_CODIGO_COMERCIO'],
                'TBK_KEY_ID': '101',
                'TBK_PARAM': self.encrypt("#".join(starmap("{}={}".format, params.items())))}

        response = requests.post(token_endpoint_url, data=data)

        r = dict(map(methodcaller('split', '='), self.decrypt(response.content.strip()).strip().split('\n')))

        if r['ERROR'] == '0':
            return r['TOKEN']


    @classmethod
    def validate(cls, request):
        commerce_code = request.POST['TBK_CODIGO_COMERCIO']  # check if ok, if not, return ERR
        decrypted_message = cls.decrypt(request.POST['TBK_PARAM'])
        params = dict(map(methodcaller('split', '='), decrypted_message.split('#')))

        answer = params['TBK_RESPUESTA']

        try:
            payment_pk = int(params['TBK_ID_SESION'])
        except:
            if (answer == u'0'):
                return cls.encrypt('ERR')
            else:
                return cls.encrypt('ACK')

        try:
            payment = getpaid_models.Payment.objects.get(pk=payment_pk,
                                          status='in_progress',
                                          backend='getpaid.backends.webpaypure')
        except getpaid_models.Payment.DoesNotExist:
            if (answer == u'0'):
                return cls.encrypt('ERR')
            else:
                return cls.encrypt('ACK')

        # create journal entry
        JournalEntry.objects.create(payment=payment, date=datetime.date.today(), body=decrypted_message)

        amount = str(int(payment.amount)) + '00'
        tbk_amount = params['TBK_MONTO']
        same_amount = amount == tbk_amount

        if (answer == u'0'):
            if same_amount and not payment.paid_on:
                payment.on_success()
                return cls.encrypt('ACK')
            else:
                payment.on_failure()
                return cls.encrypt('ERR')
        else:
            payment.on_failure()
            return cls.encrypt('ACK')

    def get_gateway_url(self, request):
        token = self.get_token()

        if self.get_backend_setting('CERTIFIED', False):
            gateway_url = 'https://webpay.transbank.cl:443/cgi-bin/bp_revision.cgi?TBK_VERSION_KCC=6.0&TBK_TOKEN={}'
        else:
            gateway_url = 'https://certificacion.webpay.cl:6443/filtroUnificado/bp_revision.cgi?TBK_VERSION_KCC=6.0&TBK_TOKEN={}'
        
        return gateway_url.format(token), "GET"



# coding: utf-8

from django.conf import settings
from django.http import HttpResponse
from django.shortcuts import render
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from getpaid.models import Payment
from . import PaymentProcessor


@require_POST
@csrf_exempt
def notify(request):
    return HttpResponse(PaymentProcessor.validate(request))


@require_POST
@csrf_exempt
def success(request):
    payment_pk = request.POST['TBK_ID_SESION']
    payment = Payment.objects.get(pk=payment_pk)
    order = payment.order
    params = payment.journalentry.params

    PAYMENT_TYPE_DESCRIPTIONS = {u'VN': u'Crédito',
                                 u'VC': u'Crédito',
                                 u'SI': u'Crédito',
                                 u'CI': u'Crédito',
                                 u'VD': u'Redcompra'}

    INSTALLMENT_TYPE_DESCRIPTIONS = {u'VN': u'Sin cuotas',
                                     u'VC': u'Cuotas normales',
                                     u'SI': u'Sin interés',
                                     u'CI': u'Cuotas comercio',
                                     u'VD': u'Venta débito'}

    context = {'payment_items': order.items,
               'order': payment.order,
               'site_url': settings.SITE_URL,
               'customer_name': order.customer_name,
               'order_id': order.pk,
               'payment_type': PAYMENT_TYPE_DESCRIPTIONS[params['TBK_TIPO_PAGO']],
               'installment_type': INSTALLMENT_TYPE_DESCRIPTIONS[params['TBK_TIPO_PAGO']],
               'last_digits': params['TBK_FINAL_NUMERO_TARJETA'],
               'transaction_date': payment.paid_on,
               'authorization_code': params['TBK_CODIGO_AUTORIZACION'],
               'amount': "{} ${}".format(payment.currency, int(payment.amount)),
               'installments': params['TBK_NUMERO_CUOTAS'].zfill(2)}
    return render(request, 'getpaid/success.html', context)


@require_POST
@csrf_exempt
def failure(request):
    context = {'order_id': request.POST['TBK_ORDEN_COMPRA']}
    return render(request, 'getpaid/failure.html', context)



from django.conf import settings
import sys


def import_name(name):
    components = name.split('.')

    if len(components) == 1:
        # direct module, import the module directly
        mod = __import__(name, globals(), locals(), [name])
    else:
        # the module is within another, so we
        # need to import it from there
        parent_path = components[0:-1]
        module_name = components[-1]

        parent_mod = __import__(
            '.'.join(parent_path), globals(), locals(), [module_name])
        mod = getattr(parent_mod, components[-1])

    return mod


def import_backend_modules(submodule=None):
    backends = getattr(settings, 'GETPAID_BACKENDS', [])
    modules = {}
    for backend_name in backends:
        fqmn = backend_name
        if submodule:
            fqmn = '%s.%s' % (fqmn, submodule)
        __import__(fqmn)
        module = sys.modules[fqmn]
        modules[backend_name] = module
    return modules


def get_backend_choices(currency=None):
    """
    Get active backends modules. Backend list can be filtered by supporting given currency.
    """
    choices = []
    backends_names = getattr(settings, 'GETPAID_BACKENDS', [])

    for backend_name in backends_names:
        backend = import_name(backend_name)
        if currency:
            if currency in backend.PaymentProcessor.BACKEND_ACCEPTED_CURRENCY:
                choices.append((backend_name, backend.PaymentProcessor.BACKEND_NAME, ))
        else:
            choices.append((backend_name, backend.PaymentProcessor.BACKEND_NAME, ))
    return choices


def get_backend_settings(backend):
    """
    Returns backend settings. If it does not exist it fails back to empty dict().
    """
    backends_settings = getattr(settings, 'GETPAID_BACKENDS_SETTINGS', {})
    try:
        return backends_settings[backend]
    except KeyError:
        return {}



from django.db import models
from django.utils.timezone import utc
from django.utils.translation import ugettext_lazy as _
from datetime import datetime
import sys
from abstract_mixin import AbstractMixin
import signals
from utils import import_backend_modules
from django.conf import settings

PAYMENT_STATUS_CHOICES = (
        ('new', _("new")),
        ('in_progress', _("in progress")),
        ('partially_paid', _("partially paid")),
        ('paid', _("paid")),
        ('failed', _("failed")),
        )


class PaymentManager(models.Manager):
    def get_query_set(self):
        return super(PaymentManager, self).get_query_set().select_related('order')


class PaymentFactory(models.Model, AbstractMixin):
    """
    This is an abstract class that defines a structure of Payment model that will be
    generated dynamically with one additional field: ``order``
    """
    amount = models.DecimalField(_("amount"), decimal_places=4, max_digits=20)
    currency = models.CharField(_("currency"), max_length=3)
    status = models.CharField(_("status"), max_length=20, choices=PAYMENT_STATUS_CHOICES, default='new', db_index=True)
    backend = models.CharField(_("backend"), max_length=50)
    created_on = models.DateTimeField(_("created on"), auto_now_add=True, db_index=True)
    paid_on = models.DateTimeField(_("paid on"), blank=True, null=True, default=None, db_index=True)
    amount_paid = models.DecimalField(_("amount paid"), decimal_places=4, max_digits=20, default=0)
    external_id = models.CharField(_("external id"), max_length=64, blank=True, null=True)
    description = models.CharField(_("description"), max_length=128, blank=True, null=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return _("Payment #%(id)d") % {'id': self.id}

    @classmethod
    def contribute(cls, order, **kwargs):
        return {'order': models.ForeignKey(order, **kwargs)}

    @classmethod
    def create(cls, order, backend):
        """
            Builds Payment object based on given Order instance
        """
        payment = Payment()
        payment.order = order
        payment.backend = backend
        signals.new_payment_query.send(sender=None, order=order, payment=payment)
        if payment.currency is None or payment.amount is None:
            raise NotImplementedError('Please provide a listener for getpaid.signals.new_payment_query')
        payment.save()
        signals.new_payment.send(sender=None, order=order, payment=payment)
        return payment

    def get_processor(self):
        try:
            __import__(self.backend)
            module = sys.modules[self.backend]
            return module.PaymentProcessor
        except (ImportError, AttributeError):
            raise ValueError("Backend '%s' is not available or provides no processor." % self.backend)

    def change_status(self, new_status):
        """
        Always change payment status via this method. Otherwise the signal
        will not be emitted.
        """
        if self.status != new_status:
            # do anything only when status is really changed
            old_status = self.status
            self.status = new_status
            self.save()
            signals.payment_status_changed.send(
                sender=type(self), instance=self,
                old_status=old_status, new_status=new_status
            )

    def on_success(self, amount=None):
        """
        Called when payment receives successful balance income. It defaults to
        complete payment, but can optionally accept received amount as a parameter
        to handle partial payments.

        Returns boolean value if payment was fully paid
        """
        if getattr(settings, 'USE_TZ', False):
            self.paid_on = datetime.utcnow().replace(tzinfo=utc)
        else:
            self.paid_on = datetime.now()
        if amount:
            self.amount_paid = amount
        else:
            self.amount_paid = self.amount
        fully_paid = (self.amount_paid >= self.amount)
        if fully_paid:
            self.change_status('paid')
        else:
            self.change_status('partially_paid')
        return fully_paid

    def on_failure(self):
        """
        Called when payment was failed
        """
        self.change_status('failed')


from django.db.models.loading import cache as app_cache, register_models
#from utils import import_backend_modules


def register_to_payment(order_class, **kwargs):
    """
    A function for registering unaware order class to ``getpaid``. This will
    generate a ``Payment`` model class that will store payments with
    ForeignKey to original order class

    This also will build a model class for every enabled backend.
    """
    global Payment
    global Order

    class Payment(PaymentFactory.construct(order=order_class, **kwargs)):
        objects = PaymentManager()

        class Meta:
            ordering = ('-created_on',)
            verbose_name = _("Payment")
            verbose_name_plural = _("Payments")

    Order = order_class

    # Now build models for backends

    backend_models_modules = import_backend_modules('models')
    for backend_name, models in backend_models_modules.items():
        app_cache.register_models(backend_name, *models.build_models(Payment))
    return Payment



from django.contrib import admin
from getpaid.models import Payment


class PaymentAdmin(admin.ModelAdmin):
    list_display = ('id', 'amount', 'currency', 'status', 'created_on', 'paid_on', 'amount_paid')
    list_filter = ('status', 'created_on', 'paid_on')
    search_fields = ('id', )
    raw_id_fields = ('order', )


admin.site.register(Payment, PaymentAdmin)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.template.base import Template
from django.template.context import Context
from getpaid.utils import get_backend_settings


class PaymentProcessorBase(object):
    """
    Base for all payment processors. It should at least be able to:
     * redirect to a gateway based on Payment object
     * manage all necessary logic to accept payment from gateway, e.g. expose
       a View for incoming transaction notification status changes
    """

    #Each backend need to define this values
    BACKEND = None
    """
    This constant should be set to fully qualified python path to the module. This is also
    a name that will be used to identify and enabling this backend in django-getpaid
    """
    BACKEND_NAME = None
    """
    This constant should be set to human readable backend name. Consider using lazy translation strings
    for i18n support
    """
    BACKEND_ACCEPTED_CURRENCY = tuple()
    """
    This constant should be any type of iterable that defines accepted currencies by the backend. Currencies should be
    set as three letters ISO code strings (eg. 'USD', 'EUR')
    """
    BACKEND_LOGO_URL = None
    """
    A path in static root where payment logo could be find.
    """

    def __init__(self, payment):

        if payment.currency not in self.BACKEND_ACCEPTED_CURRENCY:
            raise ValueError("Backend '%s' cannot process '%s' payments." % (self.BACKEND, payment.currency))
        self.payment = payment

    @classmethod
    def get_logo_url(cls):
        """
        Get backend logo. Use always this method, instead of reading BACKEND_LOGO_URL attribute directly.

        :return: str
        """
        return cls.BACKEND_LOGO_URL

    def get_order_description(self, payment, order):
        """
        Renders order description using django template provided in ``settings.GETPAID_ORDER_DESCRIPTION``
        or if not provided return unicode representation of ``Order object``.
        """
        template = getattr(settings, 'GETPAID_ORDER_DESCRIPTION', None)
        if template:
            return Template(template).render(Context({"payment": payment, "order": order}))
        else:
            return unicode(order)

    def get_gateway_url(self, request):
        """
        Should return a tuple with the first item being the URL that redirects to payment Gateway
        Second item should be if the request is made via GET or POST. Third item are the parameters
        to be passed in case of a POST REQUEST. Request context need to be given because various
        payment engines requires information about client (e.g. a client IP).
        """
        raise NotImplementedError('Must be implemented in PaymentProcessor')

    def get_form(self, post_data):
        """
        Only used if the payment processor requires POST requests.
        Generates a form only containg hidden input fields.
        """
        from getpaid.forms import PaymentHiddenInputsPostForm
        return PaymentHiddenInputsPostForm(items=post_data)

    @classmethod
    def get_backend_setting(cls, name, default=None):
        """
        Reads ``name`` setting from backend settings dictionary.

        If `default` value is omitted, raises ``ImproperlyConfigured`` when
        setting ``name`` is not available.
        """
        backend_settings = get_backend_settings(cls.BACKEND)
        if default is not None:
            return backend_settings.get(name, default)
        else:
            try:
                return backend_settings[name]
            except KeyError:
                raise ImproperlyConfigured("getpaid '%s' requires backend '%s' setting" % (cls.BACKEND, name))



from django.conf.urls import patterns, url

urlpatterns = patterns('',
    url(r'^pago/(?P<pk>\d+)/$', 'getpaid.backends.webpay.views.pago', name='getpaid-webpay-pago'),
    url(r'^resultado/(?P<pk>\d+)/$', 'getpaid.backends.webpay.views.resultado', name='getpaid-webpay-resultado'),
    url(r'^close/$', 'getpaid.backends.webpay.views.close', name='getpaid-webpay-close'),
    url(r'^success/$', 'getpaid.backends.webpay.views.success', name='getpaid-webpay-success'),
    url(r'^failure/$', 'getpaid.backends.webpay.views.failure', name='getpaid-webpay-failure'),
)



from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.views.generic.edit import FormView
from getpaid.backends.dummy import PaymentProcessor
from getpaid.backends.dummy.forms import DummyQuestionForm
from getpaid.models import Payment


class DummyAuthorizationView(FormView):
    form_class = DummyQuestionForm
    template_name = "getpaid_dummy_backend/dummy_authorization.html"

    def get_context_data(self, **kwargs):
        context = super(DummyAuthorizationView, self).get_context_data(**kwargs)
        self.payment = get_object_or_404(Payment, pk=self.kwargs['pk'], status='in_progress', backend='getpaid.backends.dummy')
        context['payment'] = self.payment
        context['order'] = self.payment.order
        context['order_name'] = PaymentProcessor(self.payment).get_order_description(self.payment, self.payment.order)  # TODO: Refactoring of get_order_description needed, should not require payment arg
        return context

    def get_success_url(self):
        url = None
        if self.success:
            url = reverse('getpaid-success-fallback', kwargs={'pk': self.payment.pk})
        else:
            url = reverse('getpaid-failure-fallback', kwargs={'pk': self.payment.pk})
        return url

    def form_valid(self, form):
        # Change payment status and jump to success_url or failure_url
        self.payment = get_object_or_404(Payment, pk=self.kwargs['pk'], status='in_progress', backend='getpaid.backends.dummy')

        if form.cleaned_data['authorize_payment'] == '1':
            self.success = True
            if not self.payment.on_success():
                # This method returns if payment was fully paid
                # if it is not, we should alert user that payment was not successfully ended anyway
                self.success = False
        else:
            self.success = False
            self.payment.on_failure()
        return super(DummyAuthorizationView, self).form_valid(form)



# Create your views here.
from django.conf import settings
from django.core.exceptions import PermissionDenied, ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.views.generic import DetailView
from django.views.generic.base import RedirectView, TemplateView
from django.views.generic.edit import FormView
from getpaid.forms import PaymentMethodForm
from getpaid.models import Payment
from getpaid.signals import redirecting_to_payment_gateway_signal


class NewPaymentView(FormView):
    form_class = PaymentMethodForm
    template_name = "getpaid/payment_post_form.html"

    def get_form(self, form_class):
        self.currency = self.kwargs['currency']
        return form_class(self.currency, **self.get_form_kwargs())

    def get(self, request, *args, **kwargs):
        """
        This view operates only on POST requests from order view where you select payment method
        """
        raise Http404

    def form_valid(self, form):
        from getpaid.models import Payment
        payment = Payment.create(form.cleaned_data['order'], form.cleaned_data['backend'])
        processor = payment.get_processor()(payment)
        gateway_url_tuple = processor.get_gateway_url(self.request)
        payment.change_status('in_progress')
        redirecting_to_payment_gateway_signal.send(sender=None, request=self.request, order=form.cleaned_data['order'], payment=payment, backend=form.cleaned_data['backend'])

        if gateway_url_tuple[1].upper() == 'GET':
            return HttpResponseRedirect(gateway_url_tuple[0])
        elif gateway_url_tuple[1].upper() == 'POST':
            context = self.get_context_data()
            context['gateway_url'] = processor.get_gateway_url(self.request)[0]
            context['form'] = processor.get_form(gateway_url_tuple[2])

            return TemplateResponse(request=self.request,
                template=self.get_template_names(),
                context=context)
        else:
            raise ImproperlyConfigured()

    def form_invalid(self, form):
        raise PermissionDenied


class FallbackView(RedirectView):
    success = None
    permanent = False

    def get_redirect_url(self, **kwargs):
        self.payment = get_object_or_404(Payment, pk=self.kwargs['pk'])

        if self.success:
            url_name = getattr(settings, 'GETPAID_SUCCESS_URL_NAME', None)
            if url_name is not None:
                return reverse(url_name, kwargs={'pk': self.payment.order_id})
        else:
            url_name = getattr(settings, 'GETPAID_FAILURE_URL_NAME', None)
            if url_name is not None:
                return reverse(url_name, kwargs={'pk': self.payment.order_id})
        return self.payment.order.get_absolute_url()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

setup(
    name='django-chile-payments',
    version='0.2.0',
    description='Chilean payment brokers for Django',
    long_description=read_md('README.md'),
    packages=find_packages(),
    url='https://github.com/Nomadblue/django-chile-payments',
    license='MIT',
    author='José Sazo',
    author_email='jose.sazo@gmail.com',
    include_package_data=True,
    install_requires=['pycrypto', 'requests', 'rsa'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from getpaid.backends import PaymentProcessorBase


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.dummy'
    BACKEND_NAME = _('Dummy backend')
    BACKEND_ACCEPTED_CURRENCY = ('PLN', 'EUR', 'USD')

    def get_gateway_url(self, request):
        return reverse('getpaid-dummy-authorization', kwargs={'pk': self.payment.pk}), "GET", {}



import urllib
from collections import OrderedDict
from itertools import starmap
from mock import Mock
from model_mommy import mommy
from django.test import TestCase
from django.utils.timezone import now
from getpaid.backends.webpay import PaymentProcessor
import getpaid


class PaymentProcessorTestCase(TestCase):
    REQUEST_PARAMS = OrderedDict((('TBK_ORDEN_COMPRA', '634185'),
                                  ('TBK_TIPO_TRANSACCION', 'TR_NORMAL'),
                                  ('TBK_RESPUESTA', '0'),
                                  ('TBK_MONTO', '11900000'),
                                  ('TBK_CODIGO_AUTORIZACION', '908315'),
                                  ('TBK_FINAL_NUMERO_TARJETA', '6623'),
                                  ('TBK_FECHA_CONTABLE', '0611'),
                                  ('TBK_FECHA_TRANSACCION', '0611'),
                                  ('TBK_HORA_TRANSACCION', '120920'),
                                  ('TBK_ID_SESION', '90'),
                                  ('TBK_ID_TRANSACCION', '250296335'),
                                  ('TBK_TIPO_PAGO', 'VN'),
                                  ('TBK_NUMERO_CUOTAS', '0'),
                                  ('TBK_VCI', 'TSY'),
                                  ('TBK_MAC', '5a778312498c624289a3b4660b8a914ee0da3770bedbbcc564b7908bb85e336fb1ee84d4c7d03ce2a0bdf12300eddb7dd06d1279ff20422e32c6736c5c6fdbe57559e89baa308f893b31d0094cb9dfc50d9f238f2db4432c4c398bd0d5fdf138c104314b1d3331223cae755732856edb307e0815868edbdd98e55dc8f2d8e5bdc7813d11353abb31f1993d6cf82a87d5ac0017974204d6090116e5c273a26392e42e6defb451c0289dd1003b743403f409a31670e6c0031a1e064ec81312b787a35046117124f28e5feb131f00c0f8f3d2f8d333237565f62f5e5767c5418d6b968b9d1b1b15abe7f6010d658ae7051a8d64737192eb43af58e7bcb2bbbea7b6a3cb4e8fcf0d0bb0576822e1bf0db744ad86ce7818961bbea81e184322a3a429abd96a6749884b7737e410fd0af4723b5628ccbd37a91ef3e37a58dff52f304aeda5ba60ac40943753871550dec8b1b09bde70f49df11059cfe757b65abd19e8e838b110ff1b0ddd6200d653db587a2ba58749cbc5f1d9108b4136b628f93401f83ead1cc835c1ec9a841cf290667e205a9b46c250d1f1932ec917499e61839422097473b0ee7b163f2b10413dd1f4f1b73713cf01797bdac20aacf9b28850127a9e24e3cbd361b460c9133d64a7c59afc41cdba92d01178e9')))

    REQUEST_BODY = "&".join(starmap("{}={}".format, REQUEST_PARAMS.items()))
    QUOTED_REQUEST_BODY = urllib.quote(REQUEST_BODY)

    def test_payment_ok(self):
        payment = mommy.make(getpaid.models.Payment, id=90, amount=119000, paid_on=None)

        request = Mock()
        request.POST = {'TBK_MONTO': '11900000'}
        request.body = self.QUOTED_REQUEST_BODY

        self.assertTrue(PaymentProcessor.validate(payment, request))

    def test_payment_diff_amount(self):
        payment = mommy.make(getpaid.models.Payment, id=90, amount=1190000, paid_on=None)

        request = Mock()
        request.POST = {'TBK_MONTO': '11900000'}
        request.body = self.QUOTED_REQUEST_BODY

        self.assertFalse(PaymentProcessor.validate(payment, request))

    def test_payment_mac_invalid(self):
        payment = mommy.make(getpaid.models.Payment, id=90, amount=119000, paid_on=None)

        request = Mock()
        request.POST = {'TBK_MONTO': '11900000'}
        request.body = self.QUOTED_REQUEST_BODY[:-1]

        self.assertFalse(PaymentProcessor.validate(payment, request))

    def test_payment_already_paid(self):
        payment = mommy.make(getpaid.models.Payment, id=90, amount=119000, paid_on=now())

        request = Mock()
        request.POST = {'TBK_MONTO': '11900000'}
        request.body = self.QUOTED_REQUEST_BODY

        self.assertFalse(PaymentProcessor.validate(payment, request))



from django.conf import settings
from django.core.exceptions import ValidationError
from django.forms import forms
from django.forms.fields import ChoiceField, CharField
from django.forms.models import ModelChoiceField
from django.forms.widgets import HiddenInput, RadioSelect, RadioFieldRenderer, RadioInput
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from getpaid.models import Order
from utils import get_backend_choices, import_name


class PaymentRadioInput(RadioInput):
    def __init__(self, name, value, attrs, choice, index):
        super(PaymentRadioInput, self).__init__(name, value, attrs, choice, index)
        logo_url = import_name(choice[0]).PaymentProcessor.get_logo_url()
        if logo_url:
            self.choice_label = mark_safe('<img src="%s%s" alt="%s">' % (
                getattr(settings, 'STATIC_URL', ''),
                logo_url,
                force_unicode(choice[1]),
                )
            )


class PaymentRadioFieldRenderer(RadioFieldRenderer):
    def __iter__(self):
        for i, choice in enumerate(self.choices):
            yield PaymentRadioInput(self.name, self.value, self.attrs.copy(), choice, i)

    def __getitem__(self, idx):
        choice = self.choices[idx]  # Let the IndexError propogate
        return PaymentRadioInput(self.name, self.value, self.attrs.copy(), choice, idx)


class PaymentRadioSelect(RadioSelect):
    renderer = PaymentRadioFieldRenderer


class PaymentMethodForm(forms.Form):
    """
    Displays all available payments backends as choice list.
    """

    def __init__(self, currency, *args, **kwargs):
        super(PaymentMethodForm, self).__init__(*args, **kwargs)
        backends = get_backend_choices(currency)
        self.fields['backend'] = ChoiceField(
            choices=backends,
            initial=backends[0][0] if len(backends) else '',
            label=_("Payment method"),
            widget=PaymentRadioSelect,
        )

    order = ModelChoiceField(widget=HiddenInput, queryset=Order.objects.all())

    def clean_order(self):
        if hasattr(self.cleaned_data['order'], 'is_ready_for_payment'):
            if not self.cleaned_data['order'].is_ready_for_payment():
                raise ValidationError(_('Order cannot be paid'))
        return self.cleaned_data['order']


class PaymentHiddenInputsPostForm(forms.Form):
    def __init__(self, items, *args, **kwargs):
        super(PaymentHiddenInputsPostForm, self).__init__(*args, **kwargs)

        for key in items:
            self.fields[key] = CharField(initial=items[key], widget=HiddenInput)
